<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Speechkit
 * @subpackage Speechkit/admin
 * @author     Kostas Vaggelakos <kostas@speechkit.io>
 * @link       https://speechkit.io
 * @since      1.0.0
 */
class Speechkit_Admin
{
    /**
     * The ID of this plugin.
     *
     * @access private
     * @var    string  $plugin_name The ID of this plugin.
     * @since  1.0.0
     */
    private $plugin_name;

    /**
     * The version of this plugin.
     *
     * @access private
     * @var    string  $version The current version of this plugin.
     * @since  1.0.0
     */
    private $version;


    /**
     * Status constants
     */
    const STATUS_NEW        = 'New';
    const STATUS_UPDATED    = 'Updated';
    const STATUS_PROCESSED  = 'Processed';
    const STATUS_PROCESSING = 'Processing';
    const STATUS_ERROR      = 'Error';
    const STATUS_FAILED     = 'Failed';
    const STATUS_SCHEDULED  = 'Scheduled';

    /**
     * Number of retries.
     */
    const RETRIES = 15;

    /**
     * Initialize the class and set its properties.
     *
     * @param string $plugin_name The name of this plugin.
     * @param string $version     The version of this plugin.
     *
     * @since 1.0.0
     */
    public function __construct($plugin_name, $version)
    {
        $options = get_option(
            'speechkit_settings',
            array(
                'speechkit_id'      => '',
            )
        );

        if (false === is_array($options) || false === array_key_exists('speechkit_id', $options)) {
            return;
        }

        $this->plugin_name = $plugin_name;

        // Using old version as a default value. All 2.5+ version will have correct
        // value present in DB, so we need to show older version just for those
        // old installs.
        $this->version = $version;

        $this->api = new SpeechkitAPI(
            $this->version,
            $options['speechkit_id']
        );
    }

    /**
     * Get all post types
     *
     * @return array
     **/
    public static function sk_available_get_post_types()
    {
        $default_ignore = array(
            'attachment',
            'revision',
            'nav_menu_item',
            'custom_css',
            'customize_changeset',
            'oembed_cache'
        );
        return array_diff(get_post_types(), $default_ignore);
    }

    /**
     * Get allowed post types
     *
     * @return array
     **/
    public static function sk_get_enabled_post_types()
    {
        $options = get_option(
            'speechkit_settings',
            array(
                'speechkit_select_post_types' => array('post'),
            )
        );

        if (!isset($options['speechkit_select_post_types']) || empty($options['speechkit_select_post_types'])) {
            return array();
        } elseif (!is_array($options['speechkit_select_post_types'])) {
            $options['speechkit_select_post_types'] = array($options['speechkit_select_post_types']);
        }

        return $options['speechkit_select_post_types'];
    }

    public function add_menu()
    {
        add_options_page(__('SpeechKit', 'speechkit'), __('SpeechKit', 'speechkit'), 'manage_options', 'speechkit', array($this, 'create_admin_interface'));
    }

    public function add_meta_box()
    {
        $post_types = self::sk_get_enabled_post_types();
        if (!empty($post_types)) {
            add_meta_box(
                'speechkit',
                __('SpeechKit', 'speechkit'),
                array($this, 'create_admin_meta_box'),
                $post_types,
                'side',
                'default'
            );
        }
    }

    /**
     * Register the stylesheets for the admin area.
     *
     * @param string $hook Page hook
     *
     * @return void
     * @since  1.0.0
     */
    public function enqueue_styles($hook)
    {
        // Always add the metabox css, because we don't know where it's added
        wp_enqueue_style($this->plugin_name . '-metabox', plugin_dir_url(__FILE__) . 'css/speechkit-admin-metabox.css', array(), $this->version, 'all');
    }

    public function setup_cron_schedules($schedules)
    {
        if (!isset($schedules['1min'])) {
            $schedules['1min'] = array(
                'interval' => 1 * 60,
                'display'  => __('Once every minute', 'speechkit'),
            );
        }
        return $schedules;
    }

    public function add_settings_link_to_plugin_page($links)
    {
        $links[] = '<a href="' .
            admin_url('options-general.php?page=speechkit') .
            '">' . __('Settings', 'speechkit') . '</a>';
        return $links;
    }

    /**
     * Register the JavaScript for the admin area.
     *
     * @param string $hook Page hook
     *
     * @return void
     * @since  1.0.0
     */
    public function enqueue_scripts($hook)
    {
        wp_register_script($this->plugin_name . '-metabox', plugin_dir_url(__FILE__) . 'js/speechkit-admin-metabox.js', array('jquery', 'underscore'), $this->version, true);

        $selected_post_types   = self::sk_get_enabled_post_types();
        $selected_post_types[] = 'post-new';

        if (in_array(str_replace('.php', '', $hook), apply_filters('spk_allowed_post_types', $selected_post_types))) {
            wp_enqueue_script($this->plugin_name . '-metabox');
            wp_localize_script(
                $this->plugin_name . '-metabox',
                'speechKitData',
                array(
                    'error'       => __("Something went wrong. This is probably SpeechKit's fault. Please try again later", 'speechkit'),
                    'enabled'     => __('Hide Audio Player', 'speechkit'),
                    'disabled'    => __('Show Audio Player', 'speechkit'),
                    'proccessing' => __('Proccesing Audio', 'speechkit'),
                    'tpl'         => array(
                        'Error'      => $this->tpl_error(),
                        'New'        => $this->tpl_new(),
                        'Processed'  => $this->tpl_processed(),
                        'Processing' => $this->tpl_processing(),
                   ),
                    'gutenberg'   => (function_exists('is_gutenberg_page') && is_gutenberg_page()),
                )
            );
        }
    }

    public function init_speechkit_block()
    {
        wp_enqueue_script(
            'speechkit-block',
            plugin_dir_url(__FILE__) . 'js/speechkit-admin.js?v='.time(),
            array('wp-blocks','wp-editor'),
            true
        );
    }

    public function create_admin_interface()
    {
        require_once plugin_dir_path(dirname(__FILE__)) . 'admin/partials/speechkit-admin-display.php';
        require_once plugin_dir_path(dirname(__FILE__)) . 'admin/partials/speechkit-admin-support.php';
    }

    public function create_admin_meta_box()
    {
        require_once plugin_dir_path(dirname(__FILE__)) . 'admin/partials/speechkit-admin-metabox.php';
    }

    public function settings_init()
    {
        register_setting(
            'speechkit',
            'speechkit_settings',
            array(
                'sanitize_callback' => array($this, 'settings_sanitize_callback'),
            )
        );

        add_settings_section(
            'speechkit_settings_section',
            __('Basic Settings', 'speechkit'),
            array($this, 'speechkit_settings_section_callback'),
            'speechkit'
        );

        add_settings_section(
            'speechkit_advanced_settings_section',
            __('Advanced Settings', 'speechkit'),
            array($this, 'speechkit_advanced_settings_section_callback'),
            'speechkit'
        );

        add_settings_field(
            'speechkit_enable',
            __('Enable SpeechKit', 'speechkit'),
            array($this, 'speechkit_enable_render'),
            'speechkit',
            'speechkit_settings_section'
        );

        add_settings_field(
            'speechkit_disable_autopublish',
            __('Disable Auto-Publish', 'speechkit'),
            array($this, 'speechkit_disable_autopublish_render'),
            'speechkit',
            'speechkit_settings_section'
        );

        add_settings_field(
            'speechkit_enable_marfeel_comp',
            __('Enable Marfeel compatibility', 'speechkit'),
            array($this, 'speechkit_enable_marfeel_comp_render'),
            'speechkit',
            'speechkit_settings_section'
        );

        add_settings_field(
            'speechkit_id',
            __('Project ID', 'speechkit'),
            array($this, 'speechkit_id_render'),
            'speechkit',
            'speechkit_settings_section'
        );

        add_settings_field(
            'speechkit_api_key',
            __('API Key', 'speechkit'),
            array($this, 'speechkit_api_key_render'),
            'speechkit',
            'speechkit_settings_section'
        );

        add_settings_field(
            'speechkit_enable_telemetry',
            __('Enable Telemetry', 'speechkit'),
            array($this, 'speechkit_enable_telemetry_render'),
            'speechkit',
            'speechkit_settings_section'
        );

        add_settings_field(
            'speechkit_merge_excerpt',
            __('Merge article excerpt with body', 'speechkit'),
            array($this, 'speechkit_merge_excerpt_render'),
            'speechkit',
            'speechkit_advanced_settings_section'
        );

        add_settings_field(
            'speechkit_hcolumn',
            __('Hide "SpeechKit status" column', 'speechkit'),
            array($this, 'speechkit_hcolumn_render'),
            'speechkit',
            'speechkit_advanced_settings_section'
        );

        add_settings_field(
            'speechkit_no_cron',
            __('Cron setting', 'speechkit'),
            array($this, 'speechkit_no_cron_render'),
            'speechkit',
            'speechkit_advanced_settings_section'
        );

        add_settings_field(
            'speechkit_select_post_types',
            __('Post types', 'speechkit'),
            array($this, 'speechkit_select_post_types_render'),
            'speechkit',
            'speechkit_advanced_settings_section'
        );
    }

    /**
     * Sanitise the SpeechKit options
     *
     * @param array $new_options The submitted options
     *
     * @return void
     * @since  2.8.0
     **/
    public function settings_sanitize_callback($new_options)
    {
        // TODO S-150 Check plugin settings allow API writes before we save them
        return $new_options;
    }

    /**
     * Perform actions after the speechkit settings have been added
     *
     * - If we have enabled Rollbar then get a new Rollbar API key
     *
     * @param array $option_name The option name
     * @param array $new_options The submitted options
     *
     * @return void
     * @since  2.8.0
     **/
    public function after_settings_added($option_name, $new_options)
    {
        if ($option_name === 'speechkit_settings') {
            $this->after_settings_updated($option_name, array(), $new_options);
        }
    }

    /**
     * Perform actions after the speechkit settings have been updated
     *
     * - If we have enabled Rollbar then get a new Rollbar API key
     *
     * @param array $option_name The option name
     * @param array $old_options The existing options
     * @param array $new_options The submitted options
     *
     * @return void
     * @since  2.8.0
     **/
    public function after_settings_updated($option_name, $old_options, $new_options)
    {
        if ($option_name !== 'speechkit_settings') {
            return;
        }

        $old_enable_telemetry = is_array($old_options) && array_key_exists('speechkit_enable_telemetry', $old_options);
        $new_enable_telemetry = is_array($new_options) && array_key_exists('speechkit_enable_telemetry', $new_options);

        if (false === $old_enable_telemetry && $new_enable_telemetry) {
            // Get & store Rollbar access token when we enable telemetry
            $rollbar_access_token = $this->api->get_rollbar_access_token();

            if (is_wp_error($rollbar_access_token)) {
                // Disable telemetry & display error notice on error
                unset($new_options['speechkit_enable_telemetry']);
                $new_options['speechkit_rollbar_error_notice'] = true;
                update_option('speechkit_settings', $new_options);
                error_log('SpeechKit: Unable to get Rollbar access token');
            } else {
                // Set Rollbar access token
                $new_options['speechkit_rollbar_access_token'] = $rollbar_access_token;
                update_option('speechkit_settings', $new_options);
            }
        } else if ($old_enable_telemetry && false === $new_enable_telemetry) {
            // Remove Rollbar access token when we disable telemetry
            unset($new_options['speechkit_rollbar_access_token']);
            update_option('speechkit_settings', $new_options);
        }
    }

    /**
     * Disable the Auto-Publish Errors setting, and clear the Rollbar access token
     *
     * In cases where the Rollbar access token has been revoked, we expect our API to
     * issue a new (valid) key when the Auto-Publish Errors setting is re-enabled
     *
     * @return void
     * @since  2.8.0
     **/
    public static function disable_auto_publish_errors()
    {
        $options = get_option('speechkit_settings', array());

        // Display
        $options['speechkit_rollbar_error_notice'] = true;

        unset($options['speechkit_enable_telemetry']);
        unset($options['speechkit_rollbar_access_token']);

        update_option('speechkit_settings', $options);
    }

    /**
     * Basic Settings section callback
     *
     * @return void
     * @since  2.5.0
     **/
    public function speechkit_settings_section_callback()
    {
    }

    /**
     * Advanced Settings section callback
     *
     * @return void
     * @since  2.5.3
     **/
    public function speechkit_advanced_settings_section_callback()
    {
    }

    /**
     * Render Enable Speechkit setting
     *
     * @return void
     * @since  2.5.0
     **/
    public function speechkit_enable_render()
    {
        $options = get_option(
            'speechkit_settings',
            array(
                'speechkit_enable' => 1,
            )
        );
        if (isset($options['speechkit_enable'])) {
            $option = $options['speechkit_enable'];
        } else {
            $option = 0;
        }
        ?>
        <input type='checkbox' name='speechkit_settings[speechkit_enable]' <?php checked($option, 1); ?> value='1'>
        <?php
    }

    /**
     * Render Hide column setting
     *
     * @return void
     * @since  2.6.7
     **/
    public function speechkit_hcolumn_render()
    {
        $options = get_option(
            'speechkit_settings',
            array(
                'speechkit_hcolumn' => 0,
            )
        );
        if (isset($options['speechkit_hcolumn'])) {
            $option = $options['speechkit_hcolumn'];
        } else {
            $option = 0;
        }
        ?>
        <input type='checkbox' name='speechkit_settings[speechkit_hcolumn]' <?php checked($option, 1); ?> value='1'>
        <?php
    }

    /**
     * Render Disable Autopublish Speechkit setting
     *
     * @return void
     * @since  2.6.2
     **/
    public function speechkit_disable_autopublish_render()
    {
        $options = get_option(
            'speechkit_settings',
            array(
                'speechkit_disable_autopublish' => 0,
            )
        );
        if (isset($options['speechkit_disable_autopublish'])) {
            $option = $options['speechkit_disable_autopublish'];
        } else {
            $option = 0;
        }
        ?>
        <input type='checkbox' name='speechkit_settings[speechkit_disable_autopublish]' <?php checked($option, 1); ?> value='1'>
        <?php _e('Disable auto-publish to generate audio for selected posts.'); ?>
        <?php
    }

    /**
     * Render Enable Marfeel Compatibility setting
     *
     * @return void
     * @since  2.6.1
     **/
    public function speechkit_enable_marfeel_comp_render()
    {
        $options = get_option(
            'speechkit_settings',
            array(
                'speechkit_enable_marfeel_comp' => 1,
            )
        );
        if (isset($options['speechkit_enable_marfeel_comp'])) {
            $option = $options['speechkit_enable_marfeel_comp'];
        } else {
            $option = 0;
        }
        ?>
        <input type='checkbox' name='speechkit_settings[speechkit_enable_marfeel_comp]' <?php checked($option, 1); ?> value='1'>
        <?php
    }

    /**
     * Render Project ID Speechkit setting
     *
     * @return void
     * @since  2.5.0
     **/
    public function speechkit_id_render()
    {
        $options = get_option('speechkit_settings');
        if (isset($options['speechkit_id'])) {
            $option = $options['speechkit_id'];
        } else {
            $option = '';
        }
        ?>
        <input type='text' name='speechkit_settings[speechkit_id]' value='<?php echo $option; ?>' size="12">
        <?php
    }

    /**
     * Render API key setting
     *
     * @return void
     * @since  2.5.0
     **/
    public function speechkit_api_key_render()
    {
        $options = get_option('speechkit_settings');
        if (isset($options['speechkit_api_key'])) {
            $option = $options['speechkit_api_key'];
        } else {
            $option = '';
        }
        ?>
        <input type='text' name='speechkit_settings[speechkit_api_key]' value='<?php echo $option; ?>' size="50">
        <?php
    }

    /**
     * Render "Enable Telemetry" setting
     *
     * - Also renders the speechkit_rollbar_access_token setting (as a hidden form field)
     *
     * @return void
     * @since  2.8.0
     **/
    public function speechkit_enable_telemetry_render()
    {
        $options = get_option('speechkit_settings', array());

        if (is_array($options) && array_key_exists('speechkit_enable_telemetry', $options)) {
            $enable_telemetry = $options['speechkit_enable_telemetry'];
        } else {
            $enable_telemetry = 0;
        }

        if (is_array($options) && array_key_exists('speechkit_rollbar_access_token', $options)) {
            $rollbar_access_token = $options['speechkit_rollbar_access_token'];
        } else {
            $rollbar_access_token = '';
        }

        ?>

        <input type="checkbox" name="speechkit_settings[speechkit_enable_telemetry]" <?php checked($enable_telemetry, 1); ?> value="1" />

        <input type="hidden" name="speechkit_settings[speechkit_rollbar_access_token]" value="<?php echo esc_attr($rollbar_access_token); ?>" />

        <?php

        printf(
            __('Automatically send diagnostic data to our <a href="%s" target="_blank">Rollbar</a> account to help us detect and fix compatibility issues, and improve our plugin.', 'speechkit'),
            'https://rollbar.com/'
        );
    }

    /**
     * Render Merge Excerpt setting
     *
     * @return void
     * @since  2.5.0
     **/
    public function speechkit_merge_excerpt_render()
    {
        $options = get_option('speechkit_settings');
        if (isset($options['speechkit_merge_excerpt'])) {
            $option = $options['speechkit_merge_excerpt'];
        } else {
            $option = 0;
        }
        ?>
        <input type='checkbox' name='speechkit_settings[speechkit_merge_excerpt]' <?php checked($option, 1); ?> value='1'>
        <?php
    }

    /**
     * Render Use Cron checkbox
     *
     * @return void
     * @since  2.5.3
     **/
    public function speechkit_no_cron_render()
    {
        $options = get_option('speechkit_settings');
        if (isset($options['speechkit_no_cron'])) {
            $option = $options['speechkit_no_cron'];
        } else {
            $option = 0;
        }
        ?>
        <input type='checkbox' name='speechkit_settings[speechkit_no_cron]' <?php checked($option, 1); ?> value='1'>
        <?php _e('Disable Cron and send post text to API immediately on post publish.'); ?>
        <p class="description">
            <?php _e('Normally posts are processed with a separate cron task. Check this if you have disabled Cron or there are issues running Cron on your server.', 'speechkit'); ?>
        </p>
        <?php
    }

    /**
     * Render Select Post Types setting
     *
     * @return void
     * @since  2.5.0
     **/
    public function speechkit_select_post_types_render()
    {
        $options = get_option(
            'speechkit_settings',
            array(
                'speechkit_select_post_types' => array(
                    'post',
                ),
            )
        );
        if (!isset($options['speechkit_select_post_types']) || empty($options['speechkit_select_post_types'])) {
            $options['speechkit_select_post_types'] = array();
        }
        ?>
        <select name='speechkit_settings[speechkit_select_post_types][]' multiple="multiple">
            <?php
            foreach (self::sk_available_get_post_types() as $key => $post_type) {
                echo '<option value="' . esc_attr($post_type) . '" ' . selected(in_array($post_type, $options['speechkit_select_post_types']), true) . '>' . esc_html($post_type) . '</option>';
            }
            ?>
        </select>
        <?php
    }

    /**
     * Get SpeechKit, WordPress, theme and plugins info
     *
     * NO NOT EXPOSE SENSITIVE DATA e.g. API keys.
     * This information may be sent over email, or without SSL.
     *
     * @return array
     * @since  2.5.0
     **/
    public static function get_environment()
    {
        // SpeechKit settings
        $settings = get_option('speechkit_settings');

        // Remove sensitive data
        unset($settings['speechkit_api_key']);

        // WordPress settings
        $bloginfo = array(
            'version'   => get_bloginfo('version'),
            'wpurl'     => get_bloginfo('wpurl'),
            'url'       => get_bloginfo('url'),
            'language'  => get_bloginfo('language'),
            'charset'   => get_bloginfo('charset'),
            'html_type' => get_bloginfo('html_type'),
            'is_rtl'    => is_rtl(),
        );

        $theme_obj = wp_get_theme();

        $theme = array(
            'name'    => $theme_obj->get('Name'),
            'uri'     => $theme_obj->get('ThemeURI'),
            'version' => $theme_obj->get('Version'),
        );

        // Merged environment settings
        $environment = array(
            'phpversion' => phpversion(),
            'settings'   => $settings,
            'bloginfo'   => $bloginfo,
            'theme'      => $theme,
            'plugins'    => array(),
        );

        // This returns a simple list
        $active_plugins = get_option('active_plugins');

        if (is_array($active_plugins) && count($active_plugins)) {
            // Check if get_plugins() function exists. This is required on the
            // front end of the site, since it is in a file that is normally
            // only loaded in the admin.
            if (!function_exists('get_plugins')) {
                require_once ABSPATH . 'wp-admin/includes/plugin.php';
            }

            // Get Plugin Version, URL, etc
            foreach (get_plugins() as $plugin_slug => $plugin) {
                if (in_array($plugin_slug, $active_plugins)) {
                    $environment['plugins'][] = array(
                        'name'    => $plugin['Name'],
                        'slug'    => $plugin_slug,
                        'version' => $plugin['Version'],
                    );
                }
            }
        }

        return apply_filters('spk_environment_data', $environment);
    }

    /**
     * Print Admin Notices
     *
     * @return void
     * @since  2.5.0
     */
    public function print_plugin_admin_notices()
    {
        $options = get_option('speechkit_settings', array());

        if (is_array($options) && array_key_exists('speechkit_rollbar_error_notice', $options)) {
            $rollbar_error_notice = $options['speechkit_rollbar_error_notice'];
        } else {
            $rollbar_error_notice = false;
        }

        // Ajax notice (initially hidden, toggled with JS)
        ?>
        <div id="speechkit-ajax-warning" class="notice notice-warning speechkit-hidden">
            <p><?php _e('A network error prevented us from checking your SpeechKit processing status. We’ll try again soon.', 'speechkit'); ?></p>
            <p><?php printf(__('If this problem persists please <i>Disable Cron</i> in the <a href="%s">SpeechKit Settings</a>.', 'speechkit'), admin_url('options-general.php?page=speechkit')); ?></p>
        </div>
        <?php

        // Email notice
        if (isset($_REQUEST['sk_form_support'])) {
            if ('success' === $_REQUEST['sk_form_support']) {
                echo '<div class="notice notice-success is-dismissible"><p>' . __('Your email was send. We will contact you shortly.', 'speechkit') . '</p></div>';
            } else {
                echo '<div class="notice notice-error is-dismissible"><p>' . __('There was an error while sending your email.', 'speechkit') . '</p></div>';
            }
        }

        if ($rollbar_error_notice) :
            ?>
                <div class="notice error">
                    <p>
                        <strong>
                            <?php
                            _e('SpeechKit Telemetry has been disabled due to an unexpected error.', 'speechkit');
                            echo '<br />';
                            _e('Please check and update the SpeechKit plugin settings, and submit a support request if this message persists.', 'speechkit');
                            ?>
                        </strong>
                    </p>
                </div>
            <?php
        endif;
    }

    public static function ensure_endbled()
    {
        $options = get_option(
            'speechkit_settings',
            array(
                'speechkit_enable' => 1,
            )
        );

        return $options['speechkit_enable'] == 1;
    }

    public static function is_disabled_autopublish()
    {
        $options = get_option('speechkit_settings');
        return isset($options['speechkit_disable_autopublish']);
    }

    public static function ensure_settings()
    {
        $options = get_option(
            'speechkit_settings',
            array(
                'speechkit_api_key' => '',
                'speechkit_id'      => '',
            )
        );

        return ($options['speechkit_api_key'] && $options['speechkit_id']);
    }

    /**
     * Handles multiple post status transitions
     *
     * @param object $new_status New status.
     * @param object $old_status Old status.
     * @param object $post       Post Object.
     *
     * @return void
     * @since  2.8.2
     **/
    public function transition_post_status($new_status, $old_status, $post)
    {
        if (false === in_array($post->post_type, self::sk_get_enabled_post_types())) {
            return;
        }

        switch ($new_status) {
            case 'draft':
                self::draft_post_hook($post->ID, $post);
                break;
            case 'publish':
            case 'pending':
            case 'private':
                self::publish_post_hook($post->ID, $post);
                break;
            case 'future':
                self::publish_post_hook($post->ID, $post, 'scheduled');
                break;
            case 'new':
            case 'auto-draft':
            case 'trash':
            case 'inherit':
                // Do nothing
                break;
            default:
                Speechkit_ErrorHandler::error(
                    'Unexpected `new_status` in `transition_post_status()`',
                    array(
                        'new_status' => $new_status,
                        'old_status' => $old_status,
                        'post' => $post,
                    )
                );
        }
    }

    public function draft_post_hook($post_id, $post)
    {
        if (self::is_elementor_ajax()) {
            return;
        }

        $status = $this->get_status($post_id);

        $statuses_to_check = array(
            self::STATUS_PROCESSED,
            self::STATUS_ERROR,
            self::STATUS_PROCESSING,
            self::STATUS_FAILED,
        );

        // Checkbox value from POST request
        $publish_checkbox = isset($_POST['publish_post_to_speechkit']) && $_POST['publish_post_to_speechkit'];

        if (in_array($status, $statuses_to_check)) {
            update_post_meta($post_id, 'publish_post_to_speechkit', true);
        } elseif (false === $publish_checkbox) {
            // TODO stop using '-1' as false for `publish_post_to_speechkit`
            update_post_meta($post_id, 'publish_post_to_speechkit', -1);
        } else {
            update_post_meta($post_id, 'publish_post_to_speechkit', true);
        }
    }

    /**
     * On post update or creation mark it to be processed later, or
     * process right away if cron is disabled.
     *
     * @param int    $post_id Post ID.
     * @param object $post    Post Object.
     *
     * @return void
     **/
    public function publish_post_hook($post_id, $post = false, $state = false)
    {
        $options = get_option('speechkit_settings');

        if (isset($options['speechkit_no_cron'])) {
            $cron_disabled = $options['speechkit_no_cron'];
        } else {
            $cron_disabled = 0;
        }

        $status = $this->get_status($post_id);

        $statuses_to_check = array(
            self::STATUS_PROCESSED,
            self::STATUS_ERROR,
            self::STATUS_SCHEDULED,
            self::STATUS_PROCESSING,
            self::STATUS_FAILED,
        );

        // Is `publish_post_to_speechkit` post_meta already set for this post?
        $publish_meta = get_post_meta($post_id, 'publish_post_to_speechkit', true);

        // If existing meta_value is '-1' then convert it to boolean false
        $publish_meta = self::negative_one_to_false($publish_meta);

        // Checkbox value from POST request
        $publish_checkbox = isset($_POST['publish_post_to_speechkit']) && $_POST['publish_post_to_speechkit'];

        switch (true) {

            case $publish_meta:                         // Already set to Publish
                break;

            case in_array($status, $statuses_to_check): // Has a Publish status
            case $publish_checkbox:                     // Checkbox is checked
                if (false === $publish_meta) {
                    update_post_meta($post_id, 'publish_post_to_speechkit', true);
                }
                break;

            case false === $publish_checkbox:           // Checkbox is unchecked
                // TODO stop using '-1' as false for `publish_post_to_speechkit`
                update_post_meta($post_id, 'publish_post_to_speechkit', -1);
                // Exit method
                return;

            default:
                Speechkit_ErrorHandler::error(
                    'publish_post_hook',
                    array(
                        'post_id' => $post_id,
                        'post' => $post,
                        'state' => $state,
                        'status' => $status,
                        'publish_meta' => $publish_meta,
                        'publish_checkbox' => $publish_checkbox,
                    )
                );
                break;
        }

        if (!$post) {
            $post = get_post($post_id);
        }

        $hash = $this->get_post_hash($post);

        switch ($status) {

            case self::STATUS_PROCESSED:
                if (get_post_meta($post_id, '_speechkit_text', true) !== $hash) {
                    update_post_meta($post_id, 'speechkit_retries', 0);
                    update_post_meta($post_id, '_speechkit_text', $hash);
                    $this->set_status($post_id, self::STATUS_UPDATED);
                    if ($cron_disabled) {
                        $this->process_post($post_id, false, $state);
                    } else {
                        if (false === wp_next_scheduled('check_speechkit_status_action')) {
                            $this->schedule_status_check();
                        }
                    }
                }
                break;

            case self::STATUS_SCHEDULED:
                $post_st = get_post_status($post_id);
                update_post_meta($post_id, 'speechkit_retries', 0);
                update_post_meta($post_id, '_speechkit_text', $hash);
                $this->set_status($post_id, self::STATUS_UPDATED);
                if ($post_st === 'publish') {
                    $state = 'unprocessed';
                } else {
                    $state = 'scheduled';
                }
                $this->process_post($post_id, true, $state);
                // The following if {} was once wrapped with...
                // if (!$cron_disabled && $post_st === 'publish') {
                if (false === wp_next_scheduled('check_speechkit_status_action')) {
                    $this->schedule_status_check();
                }
                break;

            case self::STATUS_FAILED:
            case self::STATUS_ERROR:
                Speechkit_ErrorHandler::error(
                    'Post status is failed/error',
                    array(
                        'post_id' => $post_id,
                        'post' => $post,
                        'state' => $state,
                        'status' => $status,
                        'hash' => $hash,
                        'publish_meta' => $publish_meta,
                        'publish_checkbox' => $publish_checkbox,
                    )
                );
                if (get_post_meta($post_id, '_speechkit_text', true) !== $hash) {
                    update_post_meta($post_id, 'speechkit_retries', 0);
                    update_post_meta($post_id, '_speechkit_text', $hash);
                    $this->set_status($post_id, self::STATUS_PROCESSING);
                    if ($cron_disabled) {
                        $this->process_post($post_id, false, $state);
                    } else {
                        if (false === wp_next_scheduled('check_speechkit_status_action')) {
                            $this->schedule_status_check();
                        }
                    }
                }
                break;

            case self::STATUS_PROCESSING:
                if (get_post_meta($post_id, '_speechkit_text', true) !== $hash) {
                    $this->process_post($post_id, true, $state);
                }
                break;

            default:
                update_post_meta($post_id, '_speechkit_text', $hash);
                if (false == $this->get_status($post_id)) {
                    $this->set_status($post_id, self::STATUS_NEW);
                }
                $this->process_post($post_id, false, $state);
                if ($cron_disabled) {
                    $this->process_post($post_id, false, $state);
                } else {
                    if (false === wp_next_scheduled('check_speechkit_status_action')) {
                        $this->schedule_status_check();
                    }
                }
                break;
        }
    }

    /**
     * Get hash of post title, post excerpt and post content.
     *
     * @param object $post Post object
     *
     * @return string
     * @since  2.5.3
     **/
    public function get_post_hash($post)
    {
        $title   = apply_filters('the_title', $post->post_title);
        $excerpt = '';
        $options = get_option('speechkit_settings');
        if (isset($options['speechkit_merge_excerpt']) && $options['speechkit_merge_excerpt'] == 1) {
            $excerpt = apply_filters('the_excerpt', $post->post_excerpt);
        }

        $content = apply_filters('the_content', $post->post_content);

        return wp_hash(wp_strip_all_tags($title . $excerpt . $content));
    }

    /**
     * Cron task: sends posts to Speechkit
     *
     * @return void
     * @since  2.5.0
     **/
    public function send_posts_to_speechkit()
    {
        if ($this->ensure_settings() && $this->ensure_endbled()) {
            $args = array(
                'post_type'              => self::sk_available_get_post_types(),
                'post_status'            => array(
                    'publish',
               ),
                'posts_per_page'         => 10,
                'ignore_sticky_posts'    => true,
                'cache_results'          => false,
                'update_post_term_cache' => false,
                'update_post_meta_cache' => false,
                'no_found_rows'          => true,
                'fields'                 => 'ids',
                'meta_query'             => array(
                    'relation' => 'OR',
                    array(
                        'key'     => 'speechkit_status',
                        'value'   => self::STATUS_NEW,
                        'type'    => 'CHAR',
                        'compare' => '=',
                   ),
                    array(
                        'key'     => 'speechkit_status',
                        'value'   => self::STATUS_PROCESSING,
                        'type'    => 'CHAR',
                        'compare' => '=',
                   ),
                    array(
                        'key'     => 'speechkit_status',
                        'value'   => self::STATUS_FAILED,
                        'type'    => 'CHAR',
                        'compare' => '=',
                   ),
                    array(
                        'key'     => 'speechkit_status',
                        'value'   => self::STATUS_UPDATED,
                        'type'    => 'CHAR',
                        'compare' => '=',
                   ),
               ),
            );
            $posts = new WP_Query(apply_filters('spk_posts_args_query', $args));
            if ($posts->have_posts()) {
                foreach ($posts->posts as $key => $post_id) {
                    $this->process_post($post_id);
                }
            }
        }
    }

    /**
     * Fix player for posts missed by older version.
     *
     * Cron task launched hourly. Collects posts,
     * that were previously marked as Error and
     * were stuck in that state.
     *
     * @return void
     **/
    public function process_missed_posts()
    {
        if ($this->ensure_settings() && $this->ensure_endbled()) {

            $args  = array(
                'post_type'              => self::sk_available_get_post_types(),
                'post_status'            => array(
                    'publish',
               ),
                'posts_per_page'         => 10,
                'ignore_sticky_posts'    => true,
                'cache_results'          => false,
                'update_post_term_cache' => false,
                'update_post_meta_cache' => false,
                'no_found_rows'          => true,
                'fields'                 => 'ids',
                'meta_query'             => array(
                    'relation' => 'OR',
                    array(
                        'relation' => 'AND',
                        array(
                            'key'     => 'speechkit_status',
                            'value'   => self::STATUS_ERROR,
                            'type'    => 'CHAR',
                            'compare' => '=',
                       ),
                        array(
                            'key'     => 'speechkit_retries',
                            'compare' => 'NOT EXISTS',
                            'value'   => '',
                       ),
                   ),
                    array(
                        'relation' => 'AND',
                        array(
                            'key'     => 'speechkit_status',
                            'value'   => self::STATUS_ERROR,
                            'type'    => 'CHAR',
                            'compare' => '=',
                       ),
                        array(
                            'key'     => 'speechkit_retries',
                            'value'   => self::RETRIES,
                            'type'    => 'NUMERIC',
                            'compare' => '<',
                       ),
                   ),
               ),
            );
            $posts = new WP_Query(apply_filters('spk_posts_args_query', $args));
            if ($posts->post_count <= 0) {
                wp_clear_scheduled_hook('check_speechkit_missed_action');
                return;
            }
            if ($posts->have_posts()) {
                foreach ($posts->posts as $key => $post_id) {
                    $this->process_post($post_id, true);
                }
            }
        }
    }

    /**
     * Send Post to Speechkit and proccess response
     *
     * @param int  $post_id      Post ID
     * @param bool $force_update Skip checks and forse update process.
     *
     * @return mixed
     **/
    public function process_post($post_id, $force_update = false, $state = false)
    {
        try {
            if (!intval($post_id)) {
                throw new \Exception('Invalid post_id: [' . $post_id . ']');
            }

            if (!$force_update && $this->get_retries($post_id) >= self::RETRIES) {
                $this->set_status($post_id, self::STATUS_ERROR);
                throw new \Exception('Exceeded max retries for post_id [' . $post_id . ']');
            }

            $this->set_retries($post_id, true);

            $status = $this->get_status($post_id);

            switch (true) {

                // Create any new or previously failed posts
                case empty($status):
                case $status == self::STATUS_NEW:
                case $status == self::STATUS_FAILED:
                    $response = $this->api->create_article($post_id, $state);
                    $body = json_decode(wp_remote_retrieve_body($response), true);
                    if (isset($body['errors']) && isset($body['errors']['base']) && isset($body['code']) && $body['code'] == 50) {
                        $this->set_status($post_id, self::STATUS_FAILED . __(' - You\'ve exceeded your audio quota. Please upgrade plan to create more audios', 'speechkit'));
                        return false;
                    }
                    break;

                // Update any scheduled or previously updated posts
                case $force_update:
                case $status == self::STATUS_UPDATED:
                case $status == self::STATUS_SCHEDULED:
                    $response = $this->api->update_article($post_id, $state);
                    break;

                // Get article by default
                default:
                    $response = $this->api->get_article($post_id);
                    break;

            }

            update_post_meta($post_id, 'speechkit_response', $response);

            if (is_wp_error($response)) {
                update_post_meta($post_id, 'speechkit_error', $response);
                return false;
            }

            if ($state && !empty($state) && $state == 'scheduled') {
                $this->set_status($post_id, self::STATUS_SCHEDULED);
            } else {
                $this->set_status($post_id, self::STATUS_PROCESSING);
            }

            $body = json_decode(wp_remote_retrieve_body($response), true);
            if (empty($body) || !isset($body['state']) || $body['state'] != 'processed') {
                if (isset($body['share_url']) && !empty($body['share_url'])) {
                    $this->set_player_url($post_id, $body['share_url']);
                }
                return self::STATUS_PROCESSING;
            }

            // We're done.
            $this->set_metadata_info($post_id, $body);
            $this->set_status($post_id, self::STATUS_PROCESSED);
            $this->set_retries($post_id, 0);
            $this->clear_cache($post_id);

            return self::STATUS_PROCESSED;
        } catch (\Exception $e) {
            Speechkit_ErrorHandler::error($e->getMessage(), $e);
            return false;
        }
    }

    /**
     * Clear cache for various plugins
     *
     * @param int $post_id Post ID
     *
     * @return void
     */
    public function clear_cache($post_id)
    {
        try {
            // W3TC
            if (function_exists('w3tc_flush_post')) {
                w3tc_flush_post($post_id);
            }
            // Litespeed cache
            if (function_exists('litespeed_purge_single_post')) {
                litespeed_purge_single_post($post_id);
            } elseif (class_exists('LiteSpeed_Cache')) {
                LiteSpeed_Cache::get_instance()->purge_single_post($post_id);
            }
            // Gator cache
            if (class_exists('WpGatorCache')) {
                $path = str_replace(home_url(), '', get_permalink($post_id));
                WpGatorCache::purgePath($path);
            }
            // WP Rocket cache support
            if (function_exists('rocket_clean_post')) {
                rocket_clean_post($post_id);
            }
            // WP Fastest Cache
            if (function_exists('wpfc_clear_post_cache_by_id')) {
                wpfc_clear_post_cache_by_id($post_id);
            }

            do_action('speechkit_update_post', $post_id);

        } catch (Exception $e) {
        }
    }

    /**
     * Set post status.
     *
     * @param int    $post_id Post ID
     * @param string $status  Post status
     *
     * @return string
     **/
    public function set_status($post_id, $status)
    {
        update_post_meta($post_id, 'speechkit_status', $status);

        // Clear the cache if any
        $this->clear_cache($post_id);

        return $status;
    }

    /**
     * Get post status.
     *
     * @param int $post_id Post ID
     *
     * @return string
     **/
    public function get_status($post_id)
    {
        return get_post_meta($post_id, 'speechkit_status', true);
    }

    public function set_metadata_info($post_id, $metadata)
    {
        if (isset($metadata['body']) && !empty($metadata['body'])) {
            // remove duplicate content.
            unset($metadata['body']);
        }

        update_post_meta($post_id, 'speechkit_info', $metadata);
    }

    /**
     * Get Speechkit Metadata.
     *
     * @param int $post_id Post ID.
     *
     * @return mixed
     * @since  2.5.0
     **/
    public function get_info($post_id)
    {
        return get_post_meta($post_id, 'speechkit_info', true);
    }

    public function get_retries($post_id)
    {
        $retries = get_post_meta($post_id, 'speechkit_retries', true);
        if (empty($retries)) {
            return 0;
        }
        return $retries;
    }

    /**
     * Increase the number of retries.
     *
     * @param int  $post_id  Post ID.
     * @param bool $increase True to increment retries, False to reset retries
     *
     * @return int
     * @since  2.5.0
     **/
    public function set_retries($post_id, $increase = true)
    {
        if (!$increase) {
            update_post_meta($post_id, 'speechkit_retries', 0);
            return 0;
        }

        $retries = get_post_meta($post_id, 'speechkit_retries', true);

        if (empty($retries)) {
            $retries = 0;
        }

        update_post_meta($post_id, 'speechkit_retries', ++$retries);

        return $retries;
    }

    public function clear_status_check()
    {
        wp_clear_scheduled_hook('check_speechkit_status_action');
    }

    public function schedule_status_check()
    {
        $this->clear_status_check();
        wp_schedule_event(time(), '1min', 'check_speechkit_status_action');
    }

    /**
     * Settings page actions
     */
    public function speechkit_reload_all()
    {
        if (!$this->ensure_settings() || !$this->ensure_endbled()) {
            wp_send_json_error();
        }
        $this->check_speechkit_status();
        wp_send_json_success();
    }

    /**
     * Send a support request email
     *
     * @return void
     * @since  2.5.0
     **/
    public function send_support_email()
    {
        if (isset($_POST['spk_support_nonce']) && wp_verify_nonce($_POST['spk_support_nonce'], 'sk_form_support')) {

            $to = 'support@speechkit.io';

            $message = '';
            if (isset($_POST['sk_mail_message']) && !empty($_POST['sk_mail_message'])) {
                $message = $_POST['sk_mail_message'];
            }

            $message = $message . "\n\r===Info===\n\r" . wp_json_encode(self::get_environment());
            wp_mail($to, __('Support request', 'speechkit'), $message);

            if (isset($_POST['sk_user_mail']) && !empty($_POST['sk_user_mail']) && is_email($_POST['sk_user_mail'])) {
                $message = __('Your suppport request was submited. We will contact you shortly.', 'speechkit');
                wp_mail($_POST['sk_user_mail'], __('SpeechKit support request', 'speechkit'), $message);
            } else {
                Speechkit_ErrorHandler::error('Failed to send support email', $message);
            }

            wp_redirect(
                esc_url_raw(
                    add_query_arg(
                        array(
                            'sk_form_support' => 'success',
                            'spk_response'    => 'success',
                        ),
                        admin_url('options-general.php?page=speechkit')
                    )
                )
            );
            exit;
        } else {
            Speechkit_ErrorHandler::error('Invalid `sk_form_support` nonce');
            wp_die(
                esc_html(__('Invalid nonce specified', 'speechkit')),
                esc_html(__('Error', 'speechkit')),
                array(
                    'response'  => 403,
                    'back_link' => 'options-general.php?page=speechkit',
                )
            );
        }
    }

    /***
     * Init rest api url
     */
    public function rest_api_init()
    {
        $args = array(
            'methods'  => 'POST',
            'callback' => array($this, 'speechkit_callback'),
        );
        register_rest_route('speechkit/v1', '/update_post', $args);
    }

    /**
     * Meta box actions
     */
    public function speechkit_regenerate()
    {
        if (!$this->ensure_settings() || !$this->ensure_endbled()) {
            wp_send_json_error();
        }

        $result = $this->process_post($_POST['post_id'], true);

        if ($result) {
            wp_send_json_success($result);
        } else {
            $this->schedule_status_check();
            wp_send_json_error();
        }
    }

    public function speechkit_toggle()
    {
        $post_id            = intval($_POST['post_id']);
        $current_value      = get_post_meta($post_id, 'speechkit_disabled', true);
        $speechkit_disabled = empty($current_value) ? 1 : !intval($current_value);
        if (update_post_meta($post_id, 'speechkit_disabled', $speechkit_disabled)) {
            wp_send_json_success($speechkit_disabled);
        } else {
            $this->schedule_status_check();
            wp_send_json_error();
        }
    }

    public function speechkit_reload()
    {
        if (!$this->ensure_settings() || !$this->ensure_endbled()) {
            wp_send_json_error();
        }

        $result = $this->process_post($_POST['post_id'], true);

        if ($result) {
            wp_send_json_success();
        } else {
            $this->schedule_status_check();
            wp_send_json_error();
        }
    }

    /**
     * Update player status on Edit Page pages for classic editor.
     *
     * @param array  $response  Data to return for Heartbit
     * @param array  $data      Received $_POST data
     * @param string $screen_id ID of current screen, equals to global pagenow in JS
     *
     * @return array
     * @since  2.5.0
     **/
    public function receive_heartbeat($response, $data, $screen_id)
    {
        if (!isset($data['sk-status'])) {
            if (in_array($screen_id, self::sk_get_enabled_post_types())) {
                $post_id = $data['wp-refresh-post-lock']['post_id'];

                $post_status = $this->get_status($data['wp-refresh-post-lock']['post_id']);
                if ($post_status == self::STATUS_NEW) {
                    $this->process_post($post_id);
                }

                $response['sk_status'] = $post_status;

                $info       = get_post_meta($post_id, 'speechkit_info', true);
                $audio_link = '';
                // Find the non transcoded version
                if (!empty($info) && is_array($info['media'])) {
                    foreach ($info['media'] as $media) {
                        if (isset($media['content_type']) && $media['content_type'] != 'application/x-mpegURL') {
                            $audio_link = $media['url'];
                            break;
                        }
                    }
                }

                $response['sk_data'] = array(
                    'link' => $audio_link,
                    'shown' => (1 !== get_post_meta(get_the_ID(), 'speechkit_disabled', true)),
                );
            }
        }

        return $response;
    }

    public function generate_startstop_shortcode($attr)
    {
        return '';
    }

    /**
     * Error template
     *
     * @return string
     * @since  2.5.0
     **/
    public function tpl_error()
    {
        $post_id = get_the_ID();
        $post_status = get_post_meta($post_id, 'speechkit_status', true);

        return '<p id="speechkit_status" data-status="Error">' . __('Status', 'speechkit') . ': <b>' . __('Error', 'speechkit') . '</b></p>
<p>' . __('Seems like something has gone wrong. Press \'Try Again\' to try to generate the audio again.', 'speechkit') . '</p>

<!-- Update/regenerate -->
<p>Press this to manually regenerate audio.</p>
<button type="button" class="button button-large sk-metabox-button sk_action_button" id="sk_regenerate_action" data-action="sk_regenerate">' . __('Try Again', 'speechkit') . '</button>
<hr>

<p>' . __('Having trouble? Visit our support page or email us at: <a href="mailto:support@speechkit.io">support@speechkit.io</a>', 'speechkit') . '</p>';
    }

    /**
     * New template
     *
     * @return string
     * @since  2.5.0
     **/
    public function tpl_new()
    {
        return '<p id="speechkit_status" data-status="New">' . __('Status', 'speechkit') . ': <b>' . __('New', 'speechkit') . '</b></p>
<p>SpeechKit will start producing audio shortly.</p>

<!-- Show/Hide player -->
<p>Press Hide audio player/Show audio player to remove audio from this post.</p>
<button type="button" class="button button-large sk-metabox-button sk_action_button" id="sk_toggle_action" data-action="sk_toggle">Hide Audio Player</button>
<hr>

<p>' . __('Having trouble? Visit our support page or email us at: <a href="mailto:support@speechkit.io">support@speechkit.io</a>', 'speechkit') . '</p>';
    }

    /**
     * Processed template
     *
     * @return string
     * @since  2.5.0
     **/
    public function tpl_processed()
    {
        return '<!-- Header -->
<p id="speechkit_status" data-status="Processed">' . __('Status', 'speechkit') . ': <b>' . __('Processed', 'speechkit') . '</b></p>

<!-- Listen link -->
<a href="<%- link %>" target="_blank" class="button button-large sk-metabox-button">Listen / Download</a>
<hr>

<!-- Show/Hide player -->
<p>Press Hide audio player/Show audio player to remove audio from this post.</p>
<button type="button" class="button button-large sk-metabox-button sk_action_button" id="sk_toggle_action" data-action="sk_toggle"><% if (shown) { %>Hide Audio Player<% } else { %>Show Audio Player<% } %></button>
<hr>

<!-- Update/regenerate -->
<p>Audio will generate automatically. But you can press \'Update Audio\' to regenerate manually.</p>
<button type="button" class="button button-large sk-metabox-button sk_action_button" id="sk_regenerate_action" data-action="sk_regenerate">Update Audio</button>
<hr>

<p>' . __('Having trouble? Visit our support page or email us at: <a href="mailto:support@speechkit.io">support@speechkit.io</a>', 'speechkit') . '</p>';
    }

    /**
     * Processing template
     *
     * @return string
     * @since  2.5.0
     **/
    public function tpl_processing()
    {
        return '<!-- Header -->
<p id="speechkit_status" data-status="Processing">' . __('Status', 'speechkit') . ': <b>' . __('Processing', 'speechkit') . '</b></p>

<p>SpeechKit is currently processing your post.</p>

<!-- Show/Hide player -->
<p>Press Hide audio player/Show audio player to remove audio from this post.</p>
<button type="button" class="button button-large sk-metabox-button sk_action_button" id="sk_toggle_action" data-action="sk_toggle"><% if (shown) { %>Hide Audio Player<% } else { %>Show Audio Player<% } %></button>
<hr>

<p>' . __('Having trouble? Visit our support page or email us at: <a href="mailto:support@speechkit.io">support@speechkit.io</a>', 'speechkit') . '</p>';
    }

    /**
     * Add customn column with player status.
     *
     * @param array $defaults Description
     *
     * @return array
     * @since  2.5.0
     **/
    public function posts_columns_head($defaults)
    {
        $speechkit_settings = get_option(
            'speechkit_settings',
            array(
                'speechkit_hcolumn' => 0,
            )
        );
        if (!isset($speechkit_settings['speechkit_hcolumn'])) {
            $defaults['speechkit_status'] = __('SpeechKit Status', 'speechkit');
        }
        return $defaults;
    }

    /**
     * Output player status in post list.
     *
     * @param string $column_name Column name
     * @param int    $post_id     Post ID
     *
     * @return void
     * @since  2.5.0
     **/
    public function posts_columns_content($column_name, $post_id)
    {
        if ('speechkit_status' === $column_name) {
            $status   = $this->get_status(get_the_ID());
            $disabled = get_post_meta($post_id, 'speechkit_disabled', true);

            if (!empty($status)) {
                echo esc_attr($status);

                if (!empty($disabled)) {
                    echo ', ' . __('Disabled', 'speechkit');
                }
            }
        }
    }

    /**
     * Register meta fields for rest output.
     *
     * @return void
     * @since  2.5.0
     **/
    public function register_rest_meta()
    {
        $post_types = self::sk_get_enabled_post_types();
        if (!empty($post_types)) {
            foreach ($post_types as $post_type) {
                register_meta(
                    'post',
                    'speechkit_status',
                    array(
                        'object_subtype' => $post_type,
                        'show_in_rest' => true,
                        'single' => true,
                    )
                );
            }
        }
    }

    public function speechkit_callback()
    {
        try {
            status_header(200);
            $json_str = file_get_contents('php://input');

            $json_data = json_decode($json_str, true);

            if (is_array($json_data) && array_key_exists('id', $json_data)) {
                $post_id = intval($json_data['id']);
                if ($json_data['state'] == 'processed') {
                    Speechkit_ErrorHandler::info('Callback: state == processed', array(
                        'json' => $json_data,
                        'post_id' => $post_id,
                    ));
                    if (isset($json_data['share_url']) && !empty($json_data['share_url'])) {
                        $this->set_player_url($post_id, $json_data['share_url']);
                    }
                    $this->set_metadata_info($post_id, $json_data);
                    $this->set_status($post_id, self::STATUS_PROCESSED);
                    $this->set_retries($post_id, 0);
                    $this->clear_cache($post_id);
                } else {
                    Speechkit_ErrorHandler::error('Callback: state != processed', array(
                        'json' => $json_data,
                        'post_id' => $post_id,
                    ));
                }
            } else {
                Speechkit_ErrorHandler::error('Callback: unable to parse', array(
                    'json_str' => $json_str
                ));
            }
        } catch (\Exception $e) {
            Speechkit_ErrorHandler::error('Callback Exception', $e);
        }

    }

    public function speechkit_debug()
    {
        status_header(200);
        if (isset($_POST) && isset($_POST['type']) && isset($_POST['token'])) {
            $options = get_option('speechkit_settings');
            if (isset($options['speechkit_api_key'])) {
                $api_key = $options['speechkit_api_key'];
            } else {
                $api_key = '';
            }
            if ($_POST['token'] == $api_key) {
                if ($_POST['type'] == 'post') {
                    if (isset($_POST['post'])) {
                        $post = get_post($_POST['post']);
                        var_dump($post);
                        die();
                    }
                } elseif ($_POST['type'] == 'plugins') {
                        $all_plugins = get_plugins();
                        var_dump($all_plugins);
                        die();
                } elseif ($_POST['type'] == 'plugins_active') {
                    $apl               = get_option('active_plugins');
                    $all_plugins       = get_plugins();
                    $activated_plugins = array();
                    foreach ($apl as $p) {
                        if (is_array($all_plugins) && array_key_exists($p, $all_plugins)) {
                            array_push($activated_plugins, $all_plugins[$p]);
                        }
                    }
                    var_dump($activated_plugins);
                    die();
                } elseif ($_POST['type'] == 'post_title') {
                    if (isset($_POST['post'])) {
                        $post = get_page_by_title($_POST['post']);
                        var_dump($post);
                        die();
                    }
                } elseif ($_POST['type'] == 'post_slug') {
                    if (isset($_POST['post'])) {
                        $post = get_page_by_path($_POST['post']);
                        var_dump($post);
                        die();
                    }
                } elseif ($_POST['type'] == 'post_meta') {
                    if (isset($_POST['post'])) {
                        $post_meta = get_post_meta($_POST['post']);
                        var_dump($post_meta);
                        die();
                    }
                } elseif ($_POST['type'] == 'settings') {
                    var_dump($options);
                    die();
                } else {
                    Speechkit_ErrorHandler::error('speechkit_debug() Wrong type of request');
                    echo 'Wrong type of request';
                }
            } else {
                Speechkit_ErrorHandler::error('speechkit_debug() Wrong token');
                echo 'Wrong token';
            }
        } else {
            Speechkit_ErrorHandler::error(
                'speechkit_debug() Wrong request',
                array(
                    '$_POST' => $_POST,
                )
            );
            echo 'Wrong request';
        }
    }

    /**
     * Add player link.
     *
     * @param int    $post_id Post ID.
     * @param string $link    Player URL.
     *
     * @return void
     * @since  2.5.2
     **/
    public function set_player_url($post_id, $link)
    {
        update_post_meta($post_id, '_speechkit_link', $link);
    }

    /**
     * Check constant for request
     *
     * @return void
     * @since  2.6.8
     **/
    public static function is_rest_api_request()
    {
        if (defined('REST_REQUEST') && REST_REQUEST) {
            return true;
        }
        return false;
    }

    /**
     * Convert numeric '-1' to False, because we were once storing '-1' for False
     * and in PHP '-1' evaluates as True
     *
     * @param mixed $value Any value, but we expect one of [true|false|'-1']
     *
     * @return boolean
     * @since  2.7.10
     */
    public static function negative_one_to_false($value)
    {
        // It appears that the string "-1" is always being retrieved, but
        // let's cover both numbers and strings just in case
        if ($value === -1 || $value === '-1') {
            return false;
        }
        return $value;
    }

    /**
     * Is the current request an Elementor Ajax request?
     *
     * @return boolean
     * @since  2.7.10
     */
    public static function is_elementor_ajax()
    {
        return isset($_POST['action']) && $_POST['action'] === 'elementor_ajax';
    }

}
