<?php


class SpeechkitAPI
{
    function __construct($version, $news_site_id)
    {
        $this->version      = $version;
        $this->news_site_id = $news_site_id;

        // Default to live environment
        $this->backend = 'https://app.speechkit.io';

        // Define SPEECHKIT_API_URL in wp-config.php to use other environments
        if (defined('SPEECHKIT_API_URL')) {
            $this->backend = SPEECHKIT_API_URL;
        }
    }

    private function generate_headers()
    {
        $options = get_option('speechkit_settings', array());

        if (is_array($options) && array_key_exists('speechkit_api_key', $options)) {
            $speechkit_api_key = $options['speechkit_api_key'];
        } else {
            $speechkit_api_key = '';
        }

        return array(
            'Authorization' => 'Token token=' . $speechkit_api_key,
            'Content-Type'  => 'application/json',
            'User-Agent'    => 'SpeechKit WP Plugin v' . $this->version,
        );
    }

    private function generate_default_fetch_options()
    {
        return array(
            'headers'     => $this->generate_headers(),
            'blocking'    => true,
            'timeout'     => 60,
            'httpversion' => '1.1',
        );
    }

    private function generate_body_from_post( $post_id, $state = false )
    {
        $options   = get_option( 'speechkit_settings' );
        $post      = get_post( $post_id );
        $post_meta = get_post_meta( $post_id, 'speechkit_info', true );
        // Parse out all data
        $external_id = $post_id;
        $title       = $post->post_title;

        // $body = wp_strip_all_tags( apply_filters( 'the_content', $post->post_content ) );
        $body_pre = $post->post_content;
        if ( preg_match_all('/\[SpeechKit-Start\](.*?)\[SpeechKit-Stop\]/s', $body_pre, $match, PREG_PATTERN_ORDER) > 0 ) {
            $body_pre = implode(' ', $match[1]);
        }
        $body = apply_filters( 'the_content', $body_pre );
        if( empty($body) ){
            $body = $body_pre;
        }
        // Merge the excerpt if the option is picked.
        if (is_array($options) && array_key_exists('speechkit_merge_excerpt', $options) && $options['speechkit_merge_excerpt'] == 1) {
            $body = $post->post_excerpt . '. ' . $body;
        }
        // $summary = wp_strip_all_tags(apply_filters('the_excerpt', $post->post_content ));
        $external_url = get_permalink( $post_id );
        $published_at = $post->post_date_gmt;
        $author       = get_the_author_meta( 'display_name', $post->post_author );
        $image_url    = get_the_post_thumbnail_url( $post_id, 'thumbnail' );

        //Get version in the repository
        if ( ! function_exists( 'plugins_api' ) ) {
            require_once( ABSPATH . 'wp-admin/includes/plugin-install.php' );
        }
        $args = array(
            'slug' => 'speechkit',
            'fields' => array(
                'version' => true,
            )
        );

        $call_api = plugins_api( 'plugin_information', $args );
        $latest_latest		= '';
        $publisher_version	= $this->version;
        if ( !is_wp_error( $call_api ) ) {
            if ( ! empty( $call_api->version ) ) {
                $latest_latest = $call_api->version;
            }

        }
        $body = array(
            'external_id'		=> $external_id,
            'title'        		=> $title,
            'body'         		=> $body,
            'external_url' 		=> $external_url,
            'published_at' 		=> $published_at,
            'author'       		=> $author,
            'image_url'    		=> $image_url,
            'publisher_version'	=> $publisher_version,
            'latest_version'	=> $latest_latest,
        );

        if( $state && !empty( $state ) ){
            $body['state']		= $state;
        }

        // Callback url for processing
        $body['callback_url'] = get_rest_url(null, 'speechkit/v1/update_post');

        if ( $post_meta && $post_meta['media'] ) {
            $body['media_attributes'] = $post_meta['media'];
        } else {
            $body['media_attributes'] = array(
                array(
                    'role' => 'body',
                ),
            );
        }
        return wp_json_encode($body);
    }

    private function generate_base_url()
    {
        return $this->backend . '/api/v2/projects/' . $this->news_site_id . '/';
    }

    /**
     * Process a response from wp_remote_get() or wp_remote_post()
     *
     * - Generate a Speechkit_ErrorHandler error for non-2XX responses
     *
     * Expected errors from the backend API are:
     * - 500 {code: 40, message: "Error. Please contact support@speechkit.io"}
     * - 401 {code: 20, message: "The authentication token was not recognised"}
     * - 401 {code: 60, message: "You don't have authorization to perform this action"}
     * - 422 {code: 70, message: "The audio is currently being processed, can't update now"}
     * - 402 {code: 60, error: "API access requires payment. Please enter payment information through the dashboard at: https://app.speechkit.io"}
     * - 400 {code: 30, error: "Invalid request"}
     * - 400 {code: 50, error: "Variety of validation errors"}
     * - 404 {code: 30, message: "Object not found"}
     *
     * @param mixed $response WP response object
     *
     * @since  2.8.0
     * @return mixed $response
     */
    private function _process_response($response)
    {
        $status_code = wp_remote_retrieve_response_code($response);

        // Success is always 2XX
        if ($status_code >= 200 && $status_code < 300) {
            return $response;
        }

        switch ($status_code) {
            // Expected errors
            case 500:
            case 401:
            case 422:
            case 402:
            case 400:
            case 404:
                $msg = sprintf('%d API Response', $status_code);
                Speechkit_ErrorHandler::error($msg, $response);
                break;

            // Unexpected errors
            default:
                $msg = sprintf('UNEXPECTED %d API Response', $status_code);
                Speechkit_ErrorHandler::error($msg, $response);
                break;

        }

        return $response;
    }

    public function get_article( $post_id )
    {
        $url = $this->generate_base_url() . 'podcasts/' . $post_id . '?cache=false';
        $response = wp_remote_get( $url, $this->generate_default_fetch_options() );
        return $this->_process_response($response);
    }

    public function create_article( $post_id, $state = false )
    {
        update_post_meta( $post_id, 'speechkit_req_create', $this->generate_body_from_post( $post_id, $state ) );
        $response = wp_remote_post(
            $this->generate_base_url() . 'podcasts', // $url
            array_merge(
                $this->generate_default_fetch_options(),
                array(
                    'method' => 'POST',
                    'body'   => $this->generate_body_from_post( $post_id, $state ),
                )
            )
        );
        return $this->_process_response($response);
    }

    public function update_article( $post_id, $state = 'unprocessed' )
    {
        update_post_meta( $post_id, 'speechkit_req_update', $this->generate_body_from_post( $post_id, $state ) );
        $response = wp_remote_post(
            $this->generate_base_url() . 'podcasts/' . $post_id, // $url
            array_merge(
                $this->generate_default_fetch_options(),
                array(
                    'method' => 'PUT',
                    'body'   => $this->generate_body_from_post( $post_id, $state ),
                )
            )
        );
        return $this->_process_response($response);
    }

    /**
     * GET a Rollbar access token from the SpeechKit API
     *
     * We need a Rollbar API key for error handling, and the API provides it
     * so we can revove and reissue keys without code changes.
     *
     * @since  2.8.0
     * @return void
     */
    public function get_rollbar_access_token()
    {
        $url = $this->backend . '/api/v2/rollbar';
        $response = wp_remote_get($url, $this->generate_default_fetch_options());

        $body = wp_remote_retrieve_body($response);

        if (is_wp_error($body)) {
            return $body;
        }

        // Validate response format, we expect:
        // {"rollbar_access_token": "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX"}
        $body = json_decode($body, true);
        if (is_array($body) && array_key_exists('rollbar_access_token', $body)) {
            return $body['rollbar_access_token'];
        }

        $wp_error = new WP_Error();
        $wp_error->add('speechkit_error', __('Invalid Rollbar access token response', 'speechkit'));
        return $wp_error;
    }

}
