<?php
/**
 * The file that defines the custom error handling class.
 *
 * PHP version 7.2
 *
 * @package    Speechkit
 * @subpackage Speechkit/includes
 * @author     Speechkit <info@speechkit.io>
 * @license    GPL-2.0+ <https://www.gnu.org/licenses/gpl-2.0.en.html>
 * @link       https://speechkit.io
 * @since      2.8.0
 */

use Rollbar\Rollbar;
use Rollbar\Payload\Level;

/**
 * Custom error handling using Rollbar.
 *
 * @package    Speechkit
 * @subpackage Speechkit/includes
 * @author     Speechkit <info@speechkit.io>
 * @license    GPL-2.0+ <https://www.gnu.org/licenses/gpl-2.0.en.html>
 * @link       https://speechkit.io
 * @since      2.8.0
 */
class Speechkit_ErrorHandler
{
    /**
     * Initialise Rollbar error logging.
     *
     * @static
     * @since  2.8.0
     * @return void
     */
    public static function init()
    {
        try {
            $options = get_option('speechkit_settings', array());

            // We cannot initialise Rollbar without an access token
            if (is_array($options) && array_key_exists('speechkit_rollbar_access_token', $options)) {
                $rollbar_access_token = $options['speechkit_rollbar_access_token'];
            } else {
                return;
            }

            if (empty($rollbar_access_token)) {
                return;
            }

            // Rollbar also complains if the access token is not 32 chars long
            if (false === self::is_valid_rollbar_access_token($rollbar_access_token)) {
                throw new \Exception(__('SpeechKit – Invalid Rollbar access token', 'speechkit'));
            }

            Rollbar::init(
                array(
                    'access_token' => $rollbar_access_token,
                    'code_version' => Speechkit::get_plugin_version(),
                    'custom'       => Speechkit_Admin::get_environment(),
                    'host'         => site_url(),
                    'framework'    => 'WordPress',
                ),
                false, // handle exceptions
                false, // handle errors
                false  // handle fatal errors
            );
        } catch (\Exception $e) {
            error_log($e->getMessage());
            Speechkit_Admin::disable_auto_publish_errors();
        }
    }

    /**
     * Log a message at "Info" level.
     *
     * @param string $message The message
     * @param mixed  $data    The data
     *
     * @static
     * @return void
     * @since  2.8.0
     */
    public static function info($message, $data = array())
    {
        self::log(Level::INFO, $message, $data);
    }

    /**
     * Log a message at "Error" level.
     *
     * @param string $message The message
     * @param mixed  $data    The data
     *
     * @static
     * @return void
     * @since  2.8.0
     */
    public static function error($message, $data = array())
    {
        self::log(Level::ERROR, $message, $data);
    }

    /**
     * Log a message at a given level.
     *
     * @param string $level   The error level
     * @param string $message The message
     * @param mixed  $data    The data
     *
     * @static
     * @return void
     * @since  2.8.0
     */
    private static function log($level, $message, $data)
    {
        $options = get_option('speechkit_settings', array());

        if (false === is_array($options) || false === array_key_exists('speechkit_enable_telemetry', $options)) {
            return;
        }

        if (is_array($options) && array_key_exists('speechkit_rollbar_access_token', $options)) {
            $rollbar_access_token = $options['speechkit_rollbar_access_token'];
        } else {
            $rollbar_access_token = false;
        }

        if (false === $rollbar_access_token || false === self::is_valid_rollbar_access_token($rollbar_access_token)) {
            // Log to PHP error log
            error_log($message);
            return;
        }

        // Log to Rollbar
        $response = Rollbar::log($level, $message, $data);

        // Handle 403 response for invalid/revoked Rollbar access tokens
        if ($response->getStatus() === 403) {
            if (self::refresh_access_token()) {
                // Attempt again, this time with the new access token
                self::log($level, $message, $data);
            } else {
                // Disable Error Handling
                Speechkit_Admin::disable_auto_publish_errors();
                error_log(sprintf('SpeechKit: Unable to post to Rollbar using access token [%s]', $rollbar_access_token));
                error_log($message);
            }
        }
    }

    /**
     * Validate Rollbar access token.
     *
     * Rollbar complains if our access token is not 32 chars long,
     * so let's validate it before we attempt to log anything.
     *
     * @param string $access_token The Rollbar access token
     *
     * @static
     * @return boolean
     * @since  2.8.0
     */
    private static function is_valid_rollbar_access_token($access_token)
    {
        return is_string($access_token) && strlen($access_token) === 32;
    }

    /**
     * Refresh the Rollbar access token.
     *
     * @static
     * @return boolean
     * @since  2.8.0
     */
    private static function refresh_access_token()
    {
        $options = get_option('speechkit_settings', array());

        if (is_array($options) && array_key_exists('speechkit_id', $options)) {
            $speechkit_id = $options['speechkit_id'];
        } else {
            $speechkit_id = '';
        }

        if (is_array($options) && array_key_exists('speechkit_rollbar_access_token', $options)) {
            $old_access_token = $options['speechkit_rollbar_access_token'];
        } else {
            $old_access_token = '';
        }

        $api = new SpeechkitAPI(
            Speechkit::get_plugin_version(),
            $speechkit_id
        );

        // Make API call to refresh the token
        $new_access_token = $api->get_rollbar_access_token();

        if (is_wp_error($new_access_token)) {
            Speechkit_Admin::disable_auto_publish_errors();
            error_log('SpeechKit: Unable to refresh Rollbar access token');
        }

        if ($new_access_token !== $old_access_token) {
            // Update setting
            $options['speechkit_rollbar_access_token'] = $new_access_token;
            update_option('speechkit_settings', $options);

            // Update runtime configuration for Rollbar
            Rollbar::logger()->configure(array('access_token' => $new_access_token));

            return true;
        }

        return false;
    }

}
