<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       https://speechkit.io
 * @since      1.0.0
 *
 * @package    Speechkit
 * @subpackage Speechkit/public
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Speechkit
 * @subpackage Speechkit/public
 * @author     Kostas Vaggelakos <kostas@speechkit.io>
 */
class Speechkit_Public {

    /**
     * The ID of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $plugin_name    The ID of this plugin.
     */
    private $plugin_name;

    /**
     * The version of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $version    The current version of this plugin.
     */
    private $version;

    /**
     * Initialize the class and set its properties.
     *
     * @since    1.0.0
     * @param      string    $plugin_name       The name of the plugin.
     * @param      string    $version    The version of this plugin.
     */
    public function __construct( $plugin_name, $version ) {
        $this->plugin_name = $plugin_name;
        $this->version     = $version;

        add_action( 'amp_post_template_head', array( $this, 'add_iframe_script' ) );
    }

    // public function speechkit_shortcode( $atts, $content = null ) {
    // 	return '<span class="caption">' . $content . '</span>';
    // }

    public function speechkit_content( $content ) {

        if ( ( is_singular() || Speechkit_Admin::is_rest_api_request() ) && in_array( get_post_type(), Speechkit_Admin::sk_get_enabled_post_types() ) ) {
            $status = get_post_meta( get_the_ID(), 'speechkit_status', true );
            if ( in_array( $status, array( Speechkit_Admin::STATUS_PROCESSED, Speechkit_Admin::STATUS_PROCESSING ) ) ) {
                $plugin_content = apply_filters( 'sk_the_content', $this->create_plugin_interface() );
                return $plugin_content . $content;
            }
        }

        return $content;
    }

    public function create_plugin_interface( $post_id = false ) {

        if ( ! $post_id ) {
            $post_id = get_the_ID();
        }

        if ( ! $this->can_show_player( $post_id ) ) {
            return '';
        }

        $before = '';
        $out = '';
        $after = '';

        $article = get_post_meta( $post_id, 'speechkit_info', true );
        if ( empty( $article ) || ! isset( $article['share_url'] ) ) {
            $link = get_post_meta( $post_id, '_speechkit_link', true );
            if ( ! empty( $link ) ) {
                $url = $link;
            }
        } else {
            $url = $article['share_url'];
        }

        $speechkit_settings = get_option(
            'speechkit_settings',
            array(
                'speechkit_enable_marfeel_comp' => 0,
            )
        );

        if ( isset( $url ) ) {

            $embedded_player_type = 'EmbeddedPlayer';
            $playlist_player_type = 'PlaylistPlayer';
            $iframe_player_type   = isset( $article['player_type'] ) ? $article['player_type'] : $embedded_player_type;
            $is_minimal_player    = $iframe_player_type != $embedded_player_type;
            $is_playlist_player   = $iframe_player_type == $playlist_player_type;

            if ( function_exists( 'is_amp_endpoint' ) && is_amp_endpoint() ) {
                // AMP version of player.

                $url = $url . ( parse_url( $url, PHP_URL_QUERY ) ? '&' : '?' );
                $url = apply_filters(
                    'sk_player_amp_url',
                    add_query_arg(
                        array(
                            'isAmp' => 1,
                            'ampPlayer' => $iframe_player_type,
                        ),
                        $url
                    )
                );

                $amp_image_element = '<amp-img width="643" height="150" layout="responsive" src="https://s3-eu-west-1.amazonaws.com/speechkit-js/logo.svg" placeholder></amp-img>';
                $overflow_element = '<div overflow tabindex=0 role=button aria-label="Read more">Read more!</div>';

                if ( $is_minimal_player ) {
                    $out = '<amp-iframe sandbox="allow-scripts allow-same-origin allow-popups" layout="fixed-height" frameborder="0" scrolling="no" height="43" width="auto" src="' . esc_url( $url ) . '">' . $amp_image_element . '</amp-iframe>';
                } else if ( $is_playlist_player ) {
                    global $content_width;
                    $out = '<amp-iframe sandbox="allow-scripts allow-same-origin allow-popups" layout="responsive" frameborder="0" scrolling="no" height="150" width="'. $content_width .'" src="' . esc_url( $url ) . '" resizable>' . $overflow_element . $amp_image_element . '</amp-iframe>';
                } else {
                    // NOTE: this is not duplicate
                    // with a media < 499px
                    $out = '<amp-iframe sandbox="allow-scripts allow-same-origin allow-popups" layout="fixed-height" frameborder="0" scrolling="no" media="(min-width: 499px)" height="70" width="auto" src="' . esc_url( $url ) . '">' . $amp_image_element . '</amp-iframe>';
                    // with a media > 498px
                    $out .= '<amp-iframe sandbox="allow-scripts allow-same-origin allow-popups" layout="fixed-height" frameborder="0" scrolling="no" media="(max-width: 498px)" height="140" width="auto" src="' . esc_url( $url ) . '">' . $amp_image_element . '</amp-iframe>';
                }

            } else {
                $marfeel_comp = 'style="display: none"';
                if ( isset( $speechkit_settings['speechkit_enable_marfeel_comp'] ) ) {
                    $marfeel_comp = 'height="40px" style="min-width: 280px; width: 100%; height: 40px; margin: 0px !important; border: none !important; display: block;" src="' . apply_filters( 'sk_player_url', $url ) . '"';
                }
                // default player version.
                $before = '<div class="speechkit-container" style="margin-bottom:10px">';

                $out = '<iframe id="speechkit-io-iframe" allowfullscreen="false" frameborder="0" scrolling="no" ' . $marfeel_comp .' data-src="' . apply_filters( 'sk_player_url', $url ) . '"></iframe>';

                $after = '</div>';
            }
        }

        return apply_filters( 'sk_player_before', $before ) . $out . apply_filters( 'sk_player_after', $after );
    }

    /**
     * Check if we are allowed to show player for this post
     *
     * @param int $post_id Post ID
     * @return bool
     * @since 2.5.2
     **/
    public function can_show_player( $post_id = false ) {

        if ( ! Speechkit_Admin::ensure_endbled() ) {
            return apply_filters( 'speechkit_post_player_enabled', false, $post_id );
        }

        if ( ! $post_id ) {
            if ( ! is_singular( Speechkit_Admin::sk_get_enabled_post_types() ) ) {
                return apply_filters( 'speechkit_post_player_enabled', false, $post_id );
            }
            $post_id = get_the_id();
        }

        if ( ! $post_id ) {
            return apply_filters( 'speechkit_post_player_enabled', false, $post_id );
        }

        if ( get_post_meta( $post_id, 'speechkit_disabled', true ) ) {
            return apply_filters( 'speechkit_post_player_enabled', false, $post_id );
        }

        $link = get_post_meta( $post_id, '_speechkit_link', true );
        if ( ! empty( $link ) ) {
            return apply_filters( 'speechkit_post_player_enabled', true, $post_id );
        }

        $article_data = get_post_meta( $post_id, 'speechkit_info', true );
        if ( ! empty( $article_data ) && isset( $article_data['share_url'] ) && ! empty( $article_data['share_url'] ) ) {
            return apply_filters( 'speechkit_post_player_enabled', true, $post_id );
        }

        return apply_filters( 'speechkit_post_player_enabled', false, $post_id );
    }

    /**
     * Register the stylesheets for the public-facing side of the site.
     *
     * @return void
     * @since    1.0.0
     */
    public function enqueue_styles() {
    }

    /**
     * Register the JavaScript for the public-facing side of the site.
     *
     * @return void
     * @since    1.0.0
     */
    public function enqueue_scripts() {

        wp_register_script( 'speechkit-iframe-helper', 'https://cdn.jsdelivr.net/npm/@speechkit/speechkit-audio-player@latest/dist/speechkit-iframe-helper.js', array(), null, true );

        if ( ! ( function_exists( 'is_amp_endpoint' ) && is_amp_endpoint() ) && $this->can_show_player() ) {
            wp_enqueue_script( 'speechkit-iframe-helper' );
        }
    }

    /**
     * Embed APM iframe script.
     *
     * @return void
     **/
    public function add_iframe_script() {
        if ( ! function_exists( 'is_plugin_active' ) ) {
            include_once ABSPATH . 'wp-admin/includes/plugin.php';
        }
        if ( ! is_plugin_active( 'accelerated-mobile-pages/accelerated-moblie-pages.php' ) ) {
            wp_enqueue_script( 'speechkit_amp-iframe-latest', 'https://cdn.ampproject.org/v0/amp-iframe-latest.js', array(), null, false );
        }
    }
}
